/*******************************************************************\

Module: Statement List Language Type Checking

Author: Matthias Weiss, matthias.weiss@diffblue.com

\*******************************************************************/

/// \file
/// Statement List Language Type Checking

#ifndef CPROVER_STATEMENT_LIST_STATEMENT_LIST_TYPECHECK_H
#define CPROVER_STATEMENT_LIST_STATEMENT_LIST_TYPECHECK_H

#include <util/symbol_table.h>
#include <util/typecheck.h>

#include "statement_list_parse_tree.h"

/// Create a new statement_list_typecheckt object and perform a type check to
/// fill the symbol table.
/// \param parse_tree: Parse tree generated by parsing a Statement List file.
/// \param [out] symbol_table: Object that shall be filled by this function.
///   If the symbol table is not empty when calling this function, its contents
///   are merged with the new entries.
/// \param module: Name of the file that has been parsed.
/// \param message_handler: Used to provide debug information and error
///   messages.
/// \return: False if no errors occurred, true otherwise.
bool statement_list_typecheck(
  statement_list_parse_treet &parse_tree,
  symbol_tablet &symbol_table,
  const std::string &module,
  message_handlert &message_handler);

/// Class for encapsulating the current state of the type check.
class statement_list_typecheckt : public typecheckt
{
public:
  statement_list_typecheckt(
    statement_list_parse_treet &parse_tree,
    symbol_tablet &symbol_table,
    const std::string &module,
    message_handlert &message_handler)
    : typecheckt(message_handler),
      parse_tree(parse_tree),
      symbol_table(symbol_table),
      module(module)
  {
  }

  void typecheck() override;
  void
  typecheck_function(const statement_list_parse_treet::functiont &function);
  void typecheck_function_block(
    const statement_list_parse_treet::function_blockt &function_block);

private:
  statement_list_parse_treet &parse_tree;
  // std::vector accumulator;
  symbol_tablet &symbol_table;
  const irep_idt module;

  code_typet::parameterst typecheck_fb_params(
    const statement_list_parse_treet::function_blockt &function_block);
  typet typecheck_statement_list_type(const typet &standard_text_type);
  void typecheck_statement_list_networks(
    const statement_list_parse_treet::networkst &networks);
};

#endif // CPROVER_STATEMENT_LIST_STATEMENT_LIST_TYPECHECK_H
